#include "stdafx.h"
#include "myd3d9utils.h"

D3DMATERIAL9 InitMaterial(D3DXCOLOR a, D3DXCOLOR d, D3DXCOLOR s , D3DXCOLOR e , D3DXCOLOR p )
{
	D3DMATERIAL9 mtrl;
	ZeroMemory( &mtrl, sizeof(mtrl) );
	mtrl.Ambient = a;
	mtrl.Diffuse = d;
	mtrl.Specular = s;
	mtrl.Emissive = e;
	mtrl.Power = p;
	return mtrl;
}

D3DLIGHT9 InitPointLight(D3DXCOLOR color,D3DXVECTOR3 pos)
{
	D3DLIGHT9 light;
	ZeroMemory(&light,sizeof(light));
	light.Type      = D3DLIGHT_POINT;
    light.Diffuse   = color;
    light.Specular  = color;
	light.Position = pos;
	light.Range = 100.0f;
	light.Attenuation0=1.0f;
	return light;
}

void Vec4toVec3(D3DXVECTOR3 &v3,D3DXVECTOR4 &v4)
{
	v3.x=v4.x;
	v3.y=v4.y;
	v3.z=v4.z;
}

void CreateShadowMatrix(LPDIRECT3DDEVICE9 pd3dDevice,
								D3DXMATRIXA16 *S,
								const D3DXVECTOR4 &LightPos,
								const D3DXVECTOR4 &v1,
								const D3DXVECTOR4 &v2,
								const D3DXVECTOR4 &v3)
{
	D3DXMATRIXA16 mWorld,mInv;
	D3DXPLANE plane;
	D3DXVECTOR3 v31,v32,v33;
	D3DXVECTOR4 v41,v42,v43;
	D3DXVECTOR4 lp;
	pd3dDevice->GetTransform(D3DTS_WORLD,&mWorld);
	D3DXMatrixInverse(&mInv,NULL,&mWorld);
	D3DXVec4Transform(&v41,&v1,&mWorld);
	D3DXVec4Transform(&v42,&v2,&mWorld);
	D3DXVec4Transform(&v43,&v3,&mWorld);
	Vec4toVec3(v31,v41);
	Vec4toVec3(v32,v42);
	Vec4toVec3(v33,v43);
	D3DXPlaneFromPoints(&plane,&v31,&v32,&v33);
	D3DXVec4Transform(&lp,&LightPos,&mInv);
	D3DXMatrixShadow(S, &lp, &plane);


}


extern void CreateReflectionMatrix(LPDIRECT3DDEVICE9 pd3dDevice,
									D3DXMATRIXA16 *R,
									const D3DXVECTOR4 &v1,
									const D3DXVECTOR4 &v2,
									const D3DXVECTOR4 &v3)
{
	D3DXMATRIXA16 mWorld,mInv;
	D3DXPLANE plane;
	D3DXVECTOR3 v31,v32,v33;
	D3DXVECTOR4 v41,v42,v43;

	pd3dDevice->GetTransform(D3DTS_WORLD,&mWorld);
	D3DXMatrixInverse(&mInv,NULL,&mWorld);
	D3DXVec4Transform(&v41,&v1,&mWorld);
	D3DXVec4Transform(&v42,&v2,&mWorld);
	D3DXVec4Transform(&v43,&v3,&mWorld);
	Vec4toVec3(v31,v41);
	Vec4toVec3(v32,v42);
	Vec4toVec3(v33,v43);

	D3DXPlaneFromPoints(&plane,&v31,&v32,&v33);
	D3DXMatrixReflect(R, &plane);
	
}